#define PD0 0
#define PD1 1
#define PD2 2
#define PD3 3
#define PD4 4
#define PD5 5
#define PD6 6
#define PD7 7

#define RX_CHANNELS 4

volatile uint16_t ch1Result = 127;
volatile uint16_t ch2Result = 127;
volatile uint16_t ch3Result = 127;
volatile uint16_t ch4Result = 127;

void setup() {
  Serial.begin(9600);
  
  // Set inputs
  for (uint8_t x = 2; x < RX_CHANNELS + 2; x++) {
    pinMode(x,INPUT);
  }
  
  // Attach interrupt to pin 2 as rising
  attachInterrupt(0, time_channels, RISING);

  // Reuse Timer1 to count the microseconds after pin 2, etc go high
  TCCR1A = 0;
  TIMSK1 = 0;
  TIFR1 = 0;
  TCNT1 = 0;
  TCCR1B = (1<<CS10);
}

void loop() {
  // FMS PIC 9600 format 
  // 0xF0 + Number of RX channels
  Serial.write(0xF0 + RX_CHANNELS);
  
  // 0x00
  Serial.write((byte)0x00);
  
  // The 4 channels
  Serial.write(ch1Result);
  Serial.write(ch2Result);
  Serial.write(ch3Result);
  Serial.write(ch4Result);

  // Debug: for checking the values
  /*Serial.println(ch1_result);
  Serial.println(ch2_result);
  Serial.println(ch3_result);
  Serial.println(ch4_result);
  Serial.println("-");
  delay(100);*/
}

// Check how long each channel is high for
void time_channels(void) {
  TCNT1 = 0; // Reset counter
  
  int ch1Time = 0;
  int ch2Time = 0;
  int ch3Time = 0;
  int ch4Time = 0;
  
  // While any of our 4 channels are high, check each channel and update it with the current timer value
  while ((PIND & ((1<<PD2) | (1<<PD3) | (1<<PD4) | (1<<PD5))))  { 
    if (PIND & (1<<PD2)) {    
      ch1Time = TCNT1;
    }
    if (PIND & (1<<PD3)) {    
      ch2Time = TCNT1;
    }
    if (PIND & (1<<PD4)) {    
      ch3Time = TCNT1;
    }
    if (PIND & (1<<PD5)) {    
      ch4Time = TCNT1;
    }
  }
  
  // Convert timer values to ~950 - 2000uS times
  ch1Time = ch1Time / 16;
  ch2Time = ch2Time / 16;
  ch3Time = ch3Time / 16;
  ch4Time = ch4Time / 16;
  
  // Map the times to 0 to 255
  // Adjust these values so your endpoints correspond close to 0 if you move the stick one way and close to 255 the other way
  ch1Result = map(ch1Time, 980, 2020, 0, 255);
  ch2Result = map(ch2Time, 980, 1994, 0, 255);
  ch3Result = map(ch3Time, 980, 1994, 0, 255);
  ch4Result = map(ch4Time, 980, 2000, 0, 255);
}
